@@ -13,15 +13,15 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: arduino.h 808 2009-02-25 09:39:04Z joerg_wunsch $ */
+/* $Id: arduino.h 1107 2012-11-20 14:03:50Z joerg_wunsch $ */
 
 #ifndef arduino_h__
 #define arduino_h__
 
+extern const char arduino_desc[];
 void arduino_initpgm (PROGRAMMER * pgm);
 
 #endif
