@@ -13,11 +13,10 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: fileio.c 964 2011-08-12 15:53:53Z joerg_wunsch $ */
+/* $Id: fileio.c 1216 2013-09-11 09:21:37Z joerg_wunsch $ */
 
 #include "ac_cfg.h"
 
@@ -27,6 +26,16 @@
 #include <string.h>
 #include <errno.h>
 #include <ctype.h>
+#include <stdint.h>
+
+#ifdef HAVE_LIBELF
+#ifdef HAVE_LIBELF_H
+#include <libelf.h>
+#elif defined(HAVE_LIBELF_LIBELF_H)
+#include <libelf/libelf.h>
+#endif
+#define EM_AVR32 0x18ad         /* inofficial */
+#endif
 
 #include "avrdude.h"
 #include "avr.h"
@@ -52,30 +61,40 @@ static int b2ihex(unsigned char * inbuf, int bufsize,
              char * outfile, FILE * outf);
 
 static int ihex2b(char * infile, FILE * inf,
-             unsigned char * outbuf, int bufsize);
+             AVRMEM * mem, int bufsize, unsigned int fileoffset);
 
 static int b2srec(unsigned char * inbuf, int bufsize, 
            int recsize, int startaddr,
            char * outfile, FILE * outf);
 
 static int srec2b(char * infile, FILE * inf,
-             unsigned char * outbuf, int bufsize);
+             AVRMEM * mem, int bufsize, unsigned int fileoffset);
 
 static int ihex_readrec(struct ihexrec * ihex, char * rec);
 
 static int srec_readrec(struct ihexrec * srec, char * rec);
 
 static int fileio_rbin(struct fioparms * fio,
-                  char * filename, FILE * f, unsigned char * buf, int size);
+                  char * filename, FILE * f, AVRMEM * mem, int size);
 
 static int fileio_ihex(struct fioparms * fio, 
-                  char * filename, FILE * f, unsigned char * buf, int size);
+                  char * filename, FILE * f, AVRMEM * mem, int size);
 
 static int fileio_srec(struct fioparms * fio,
-                  char * filename, FILE * f, unsigned char * buf, int size);
+                  char * filename, FILE * f, AVRMEM * mem, int size);
+
+#ifdef HAVE_LIBELF
+static int elf2b(char * infile, FILE * inf,
+                 AVRMEM * mem, struct avrpart * p,
+                 int bufsize, unsigned int fileoffset);
+
+static int fileio_elf(struct fioparms * fio,
+                      char * filename, FILE * f, AVRMEM * mem,
+                      struct avrpart * p, int size);
+#endif
 
 static int fileio_num(struct fioparms * fio,
-		char * filename, FILE * f, unsigned char * buf, int size,
+		char * filename, FILE * f, AVRMEM * mem, int size,
 		FILEFMT fmt);
 
 static int fmt_autodetect(char * fname);
@@ -89,6 +108,7 @@ char * fmtstr(FILEFMT format)
     case FMT_SREC : return "Motorola S-Record"; break;
     case FMT_IHEX : return "Intel Hex"; break;
     case FMT_RBIN : return "raw binary"; break;
+    case FMT_ELF  : return "ELF"; break;
     default       : return "invalid format"; break;
   };
 }
@@ -262,23 +282,20 @@ static int ihex_readrec(struct ihexrec * ihex, char * rec)
  *
  * */
 static int ihex2b(char * infile, FILE * inf,
-             unsigned char * outbuf, int bufsize)
+             AVRMEM * mem, int bufsize, unsigned int fileoffset)
 {
   char buffer [ MAX_LINE_LEN ];
-  unsigned char * buf;
-  unsigned int nextaddr, baseaddr, maxaddr, offsetaddr;
+  unsigned int nextaddr, baseaddr, maxaddr;
   int i;
   int lineno;
   int len;
   struct ihexrec ihex;
   int rc;
 
-  lineno      = 0;
-  buf         = outbuf;
-  baseaddr    = 0;
-  maxaddr     = 0;
-  offsetaddr  = 0;
-  nextaddr    = 0;
+  lineno   = 0;
+  baseaddr = 0;
+  maxaddr  = 0;
+  nextaddr = 0;
 
   while (fgets((char *)buffer,MAX_LINE_LEN,inf)!=NULL) {
     lineno++;
@@ -303,22 +320,29 @@ static int ihex2b(char * infile, FILE * inf,
 
     switch (ihex.rectyp) {
       case 0: /* data record */
-        nextaddr = ihex.loadofs + baseaddr;
-        if ((nextaddr + ihex.reclen) > (bufsize+offsetaddr)) {
+        if (fileoffset != 0 && baseaddr < fileoffset) {
+          fprintf(stderr, 
+                  "%s: ERROR: address 0x%04x out of range (below fileoffset 0x%x) at line %d of %s\n",
+                  progname, baseaddr, fileoffset, lineno, infile);
+          return -1;
+        }
+        nextaddr = ihex.loadofs + baseaddr - fileoffset;
+        if (nextaddr + ihex.reclen > bufsize) {
           fprintf(stderr, 
                   "%s: ERROR: address 0x%04x out of range at line %d of %s\n",
                   progname, nextaddr+ihex.reclen, lineno, infile);
           return -1;
         }
         for (i=0; i<ihex.reclen; i++) {
-          buf[nextaddr+i-offsetaddr] = ihex.data[i];
+          mem->buf[nextaddr+i] = ihex.data[i];
+          mem->tags[nextaddr+i] = TAG_ALLOCATED;
         }
         if (nextaddr+ihex.reclen > maxaddr)
           maxaddr = nextaddr+ihex.reclen;
         break;
 
       case 1: /* end of file record */
-        return maxaddr-offsetaddr;
+        return maxaddr;
         break;
 
       case 2: /* extended segment address record */
@@ -331,7 +355,6 @@ static int ihex2b(char * infile, FILE * inf,
 
       case 4: /* extended linear address record */
         baseaddr = (ihex.data[0] << 8 | ihex.data[1]) << 16;
-        if(nextaddr == 0) offsetaddr = baseaddr;	// if provided before any data, then remember it
         break;
 
       case 5: /* start linear address record */
@@ -349,12 +372,22 @@ static int ihex2b(char * infile, FILE * inf,
 
   } /* while */
 
-  fprintf(stderr, 
-          "%s: WARNING: no end of file record found for Intel Hex "
-          "file \"%s\"\n",
-          progname, infile);
+  if (maxaddr == 0) {
+    fprintf(stderr, 
+	    "%s: ERROR: No valid record found in Intel Hex "
+	    "file \"%s\"\n",
+	    progname, infile);
 
-  return maxaddr-offsetaddr;
+    return -1;
+  }
+  else {
+    fprintf(stderr, 
+	    "%s: WARNING: no end of file record found for Intel Hex "
+	    "file \"%s\"\n",
+	    progname, infile);
+
+    return maxaddr;
+  }
 }
 
 static int b2srec(unsigned char * inbuf, int bufsize, 
@@ -540,11 +573,10 @@ static int srec_readrec(struct ihexrec * srec, char * rec)
 
 
 static int srec2b(char * infile, FILE * inf,
-           unsigned char * outbuf, int bufsize)
+           AVRMEM * mem, int bufsize, unsigned int fileoffset)
 {
   char buffer [ MAX_LINE_LEN ];
-  unsigned char * buf;
-  unsigned int nextaddr, baseaddr, maxaddr;
+  unsigned int nextaddr, maxaddr;
   int i;
   int lineno;
   int len;
@@ -556,8 +588,6 @@ static int srec2b(char * infile, FILE * inf,
   char * msg = 0;
 
   lineno   = 0;
-  buf      = outbuf;
-  baseaddr = 0;
   maxaddr  = 0;
   reccount = 0;
 
@@ -591,17 +621,17 @@ static int srec2b(char * infile, FILE * inf,
 
       case 0x31: /* S1 - 16 bit address data record */
         datarec=1;
-        msg="%s: ERROR: address 0x%04x out of range at line %d of %s\n";    
+        msg="%s: ERROR: address 0x%04x out of range %sat line %d of %s\n";
         break;
 
       case 0x32: /* S2 - 24 bit address data record */
         datarec=1;
-        msg="%s: ERROR: address 0x%06x out of range at line %d of %s\n";
+        msg="%s: ERROR: address 0x%06x out of range %sat line %d of %s\n";
         break;
 
       case 0x33: /* S3 - 32 bit address data record */
         datarec=1;
-        msg="%s: ERROR: address 0x%08x out of range at line %d of %s\n";
+        msg="%s: ERROR: address 0x%08x out of range %sat line %d of %s\n";
         break;
 
       case 0x34: /* S4 - symbol record (LSI extension) */
@@ -637,13 +667,23 @@ static int srec2b(char * infile, FILE * inf,
     }
 
     if (datarec == 1) {
-      nextaddr = srec.loadofs + baseaddr;
-      if (nextaddr + srec.reclen > bufsize) {
-        fprintf(stderr, msg, progname, nextaddr+srec.reclen, lineno, infile);
+      nextaddr = srec.loadofs;
+      if (nextaddr < fileoffset) {
+        fprintf(stderr, msg, progname, nextaddr,
+                "(below fileoffset) ",
+                lineno, infile);
         return -1;
       }
-      for (i=0; i<srec.reclen; i++) 
-        buf[nextaddr+i] = srec.data[i];
+      nextaddr -= fileoffset;
+      if (nextaddr + srec.reclen > bufsize) {
+        fprintf(stderr, msg, progname, nextaddr+srec.reclen, "",
+                lineno, infile);
+        return -1;
+      }
+      for (i=0; i<srec.reclen; i++) {
+        mem->buf[nextaddr+i] = srec.data[i];
+        mem->tags[nextaddr+i] = TAG_ALLOCATED;
+      }
       if (nextaddr+srec.reclen > maxaddr)
         maxaddr = nextaddr+srec.reclen;
       reccount++;	
@@ -659,6 +699,385 @@ static int srec2b(char * infile, FILE * inf,
   return maxaddr;
 }
 
+#ifdef HAVE_LIBELF
+/*
+ * Determine whether the ELF file section pointed to by `sh' fits
+ * completely into the program header segment pointed to by `ph'.
+ *
+ * Assumes the section has been checked already before to actually
+ * contain data (SHF_ALLOC, SHT_PROGBITS, sh_size > 0).
+ *
+ * Sometimes, program header segments might be larger than the actual
+ * file sections.  On VM architectures, this is used to allow mmapping
+ * the entire ELF file "as is" (including things like the program
+ * header table itself).
+ */
+static inline
+int is_section_in_segment(Elf32_Shdr *sh, Elf32_Phdr *ph)
+{
+    if (sh->sh_offset < ph->p_offset)
+        return 0;
+    if (sh->sh_offset + sh->sh_size > ph->p_offset + ph->p_filesz)
+        return 0;
+    return 1;
+}
+
+/*
+ * Return the ELF section descriptor that corresponds to program
+ * header `ph'.  The program header is expected to be of p_type
+ * PT_LOAD, and to have a nonzero p_filesz.  (PT_LOAD sections with a
+ * zero p_filesz are typically RAM sections that are not initialized
+ * by file data, e.g. ".bss".)
+ */
+static Elf_Scn *elf_get_scn(Elf *e, Elf32_Phdr *ph, Elf32_Shdr **shptr)
+{
+  Elf_Scn *s = NULL;
+
+  while ((s = elf_nextscn(e, s)) != NULL) {
+    Elf32_Shdr *sh;
+    size_t ndx = elf_ndxscn(s);
+    if ((sh = elf32_getshdr(s)) == NULL) {
+      fprintf(stderr,
+              "%s: ERROR: Error reading section #%u header: %s\n",
+              progname, (unsigned int)ndx, elf_errmsg(-1));
+      continue;
+    }
+    if ((sh->sh_flags & SHF_ALLOC) == 0 ||
+        sh->sh_type != SHT_PROGBITS)
+      /* we are only interested in PROGBITS, ALLOC sections */
+      continue;
+    if (sh->sh_size == 0)
+      /* we are not interested in empty sections */
+      continue;
+    if (is_section_in_segment(sh, ph)) {
+      /* yeah, we found it */
+      *shptr = sh;
+      return s;
+    }
+  }
+
+  fprintf(stderr,
+          "%s: ERROR: Cannot find a matching section for "
+          "program header entry @p_vaddr 0x%x\n",
+          progname, ph->p_vaddr);
+  return NULL;
+}
+
+static int elf_mem_limits(AVRMEM *mem, struct avrpart * p,
+                          unsigned int *lowbound,
+                          unsigned int *highbound,
+                          unsigned int *fileoff)
+{
+  int rv = 0;
+
+  if (p->flags & AVRPART_AVR32) {
+    if (strcmp(mem->desc, "flash") == 0) {
+      *lowbound = 0x80000000;
+      *highbound = 0xffffffff;
+      *fileoff = 0;
+    } else {
+      rv = -1;
+    }
+  } else {
+    if (strcmp(mem->desc, "flash") == 0 ||
+        strcmp(mem->desc, "boot") == 0 ||
+        strcmp(mem->desc, "application") == 0 ||
+        strcmp(mem->desc, "apptable") == 0) {
+      *lowbound = 0;
+      *highbound = 0x7ffff;       /* max 8 MiB */
+      *fileoff = 0;
+    } else if (strcmp(mem->desc, "eeprom") == 0) {
+      *lowbound = 0x810000;
+      *highbound = 0x81ffff;      /* max 64 KiB */
+      *fileoff = 0;
+    } else if (strcmp(mem->desc, "lfuse") == 0) {
+      *lowbound = 0x820000;
+      *highbound = 0x82ffff;
+      *fileoff = 0;
+    } else if (strcmp(mem->desc, "hfuse") == 0) {
+      *lowbound = 0x820000;
+      *highbound = 0x82ffff;
+      *fileoff = 1;
+    } else if (strcmp(mem->desc, "efuse") == 0) {
+      *lowbound = 0x820000;
+      *highbound = 0x82ffff;
+      *fileoff = 2;
+    } else if (strncmp(mem->desc, "fuse", 4) == 0 &&
+               (mem->desc[4] >= '0' && mem->desc[4] <= '9')) {
+      /* Xmega fuseN */
+      *lowbound = 0x820000;
+      *highbound = 0x82ffff;
+      *fileoff = mem->desc[4] - '0';
+    } else if (strcmp(mem->desc, "lock") == 0) {
+      *lowbound = 0x830000;
+      *highbound = 0x83ffff;
+      *fileoff = 0;
+    } else {
+      rv = -1;
+    }
+  }
+
+  return rv;
+}
+
+
+static int elf2b(char * infile, FILE * inf,
+                 AVRMEM * mem, struct avrpart * p,
+                 int bufsize, unsigned int fileoffset)
+{
+  Elf *e;
+  int rv = -1;
+  unsigned int low, high, foff;
+
+  if (elf_mem_limits(mem, p, &low, &high, &foff) != 0) {
+    fprintf(stderr,
+            "%s: ERROR: Cannot handle \"%s\" memory region from ELF file\n",
+            progname, mem->desc);
+    return -1;
+  }
+
+  /*
+   * The Xmega memory regions for "boot", "application", and
+   * "apptable" are actually sub-regions of "flash".  Refine the
+   * applicable limits.  This allows to select only the appropriate
+   * sections out of an ELF file that contains section data for more
+   * than one sub-segment.
+   */
+  if ((p->flags & AVRPART_HAS_PDI) != 0 &&
+      (strcmp(mem->desc, "boot") == 0 ||
+       strcmp(mem->desc, "application") == 0 ||
+       strcmp(mem->desc, "apptable") == 0)) {
+    AVRMEM *flashmem = avr_locate_mem(p, "flash");
+    if (flashmem == NULL) {
+      fprintf(stderr,
+              "%s: ERROR: No \"flash\" memory region found, "
+              "cannot compute bounds of \"%s\" sub-region.\n",
+              progname, mem->desc);
+      return -1;
+    }
+    /* The config file offsets are PDI offsets, rebase to 0. */
+    low = mem->offset - flashmem->offset;
+    high = low + mem->size - 1;
+  }
+
+  if (elf_version(EV_CURRENT) == EV_NONE) {
+    fprintf(stderr,
+            "%s: ERROR: ELF library initialization failed: %s\n",
+            progname, elf_errmsg(-1));
+    return -1;
+  }
+  if ((e = elf_begin(fileno(inf), ELF_C_READ, NULL)) == NULL) {
+    fprintf(stderr,
+            "%s: ERROR: Cannot open \"%s\" as an ELF file: %s\n",
+            progname, infile, elf_errmsg(-1));
+    return -1;
+  }
+  if (elf_kind(e) != ELF_K_ELF) {
+    fprintf(stderr,
+            "%s: ERROR: Cannot use \"%s\" as an ELF input file\n",
+            progname, infile);
+    goto done;
+  }
+
+  size_t i, isize;
+  const char *id = elf_getident(e, &isize);
+
+  if (id == NULL) {
+    fprintf(stderr,
+            "%s: ERROR: Error reading ident area of \"%s\": %s\n",
+            progname, infile, elf_errmsg(-1));
+    goto done;
+  }
+
+  const char *endianname;
+  unsigned char endianess;
+  if (p->flags & AVRPART_AVR32) {
+    endianess = ELFDATA2MSB;
+    endianname = "little";
+  } else {
+    endianess = ELFDATA2LSB;
+    endianname = "big";
+  }
+  if (id[EI_CLASS] != ELFCLASS32 ||
+      id[EI_DATA] != endianess) {
+    fprintf(stderr,
+            "%s: ERROR: ELF file \"%s\" is not a "
+            "32-bit, %s-endian file that was expected\n",
+            progname, infile, endianname);
+    goto done;
+  }
+
+  Elf32_Ehdr *eh;
+  if ((eh = elf32_getehdr(e)) == NULL) {
+    fprintf(stderr,
+            "%s: ERROR: Error reading ehdr of \"%s\": %s\n",
+            progname, infile, elf_errmsg(-1));
+    goto done;
+  }
+
+  if (eh->e_type != ET_EXEC) {
+    fprintf(stderr,
+            "%s: ERROR: ELF file \"%s\" is not an executable file\n",
+            progname, infile);
+    goto done;
+  }
+
+  const char *mname;
+  uint16_t machine;
+  if (p->flags & AVRPART_AVR32) {
+    machine = EM_AVR32;
+    mname = "AVR32";
+  } else {
+    machine = EM_AVR;
+    mname = "AVR";
+  }
+  if (eh->e_machine != machine) {
+    fprintf(stderr,
+            "%s: ERROR: ELF file \"%s\" is not for machine %s\n",
+            progname, infile, mname);
+    goto done;
+  }
+  if (eh->e_phnum == 0xffff /* PN_XNUM */) {
+    fprintf(stderr,
+            "%s: ERROR: ELF file \"%s\" uses extended "
+            "program header numbers which are not expected\n",
+            progname, infile);
+    goto done;
+  }
+
+  Elf32_Phdr *ph;
+  if ((ph = elf32_getphdr(e)) == NULL) {
+    fprintf(stderr,
+            "%s: ERROR: Error reading program header table of \"%s\": %s\n",
+            progname, infile, elf_errmsg(-1));
+    goto done;
+  }
+
+  size_t sndx;
+  if (elf_getshdrstrndx(e, &sndx) != 0) {
+    fprintf(stderr,
+            "%s: ERROR: Error obtaining section name string table: %s\n",
+            progname, elf_errmsg(-1));
+    sndx = 0;
+  }
+
+  /*
+   * Walk the program header table, pick up entries that are of type
+   * PT_LOAD, and have a non-zero p_filesz.
+   */
+  for (i = 0; i < eh->e_phnum; i++) {
+    if (ph[i].p_type != PT_LOAD ||
+        ph[i].p_filesz == 0)
+      continue;
+
+    if (verbose >= 2) {
+      fprintf(stderr,
+              "%s: Considering PT_LOAD program header entry #%d:\n"
+              "    p_vaddr 0x%x, p_paddr 0x%x, p_filesz %d\n",
+              progname, i, ph[i].p_vaddr, ph[i].p_paddr, ph[i].p_filesz);
+    }
+
+    Elf32_Shdr *sh;
+    Elf_Scn *s = elf_get_scn(e, ph + i, &sh);
+    if (s == NULL)
+      continue;
+
+    if ((sh->sh_flags & SHF_ALLOC) && sh->sh_size) {
+      const char *sname;
+
+      if (sndx != 0) {
+        sname = elf_strptr(e, sndx, sh->sh_name);
+      } else {
+        sname = "*unknown*";
+      }
+
+      unsigned int lma;
+      lma = ph[i].p_paddr + sh->sh_offset - ph[i].p_offset;
+
+      if (verbose >= 2) {
+        fprintf(stderr,
+                "%s: Found section \"%s\", LMA 0x%x, sh_size %u\n",
+                progname, sname, lma, sh->sh_size);
+      }
+
+      if (lma >= low &&
+          lma + sh->sh_size < high) {
+        /* OK */
+      } else {
+        if (verbose >= 2) {
+          fprintf(stderr,
+                  "    => skipping, inappropriate for \"%s\" memory region\n",
+                  mem->desc);
+        }
+        continue;
+      }
+      /*
+       * 1-byte sized memory regions are special: they are used for fuse
+       * bits, where multiple regions (in the config file) map to a
+       * single, larger region in the ELF file (e.g. "lfuse", "hfuse",
+       * and "efuse" all map to ".fuse").  We silently accept a larger
+       * ELF file region for these, and extract the actual byte to write
+       * from it, using the "foff" offset obtained above.
+       */
+      if (mem->size != 1 &&
+          sh->sh_size > mem->size) {
+        fprintf(stderr,
+                "%s: ERROR: section \"%s\" does not fit into \"%s\" memory:\n"
+                "    0x%x + %u > %u\n",
+                progname, sname, mem->desc,
+                lma, sh->sh_size, mem->size);
+        continue;
+      }
+
+      Elf_Data *d = NULL;
+      while ((d = elf_getdata(s, d)) != NULL) {
+        if (verbose >= 2) {
+          fprintf(stderr,
+                  "    Data block: d_buf %p, d_off 0x%x, d_size %d\n",
+                  d->d_buf, (unsigned int)d->d_off, d->d_size);
+        }
+        if (mem->size == 1) {
+          if (d->d_off != 0) {
+            fprintf(stderr,
+                    "%s: ERROR: unexpected data block at offset != 0\n",
+                    progname);
+          } else if (foff >= d->d_size) {
+            fprintf(stderr,
+                    "%s: ERROR: ELF file section does not contain byte at offset %d\n",
+                    progname, foff);
+          } else {
+            if (verbose >= 2) {
+              fprintf(stderr,
+                      "    Extracting one byte from file offset %d\n",
+                      foff);
+            }
+            mem->buf[0] = ((unsigned char *)d->d_buf)[foff];
+            mem->tags[0] = TAG_ALLOCATED;
+            rv = 1;
+          }
+        } else {
+          unsigned int idx;
+
+          idx = lma - low + d->d_off;
+          if ((int)(idx + d->d_size) > rv)
+            rv = idx + d->d_size;
+          if (verbose >= 3) {
+            fprintf(stderr,
+                    "    Writing %d bytes to mem offset 0x%x\n",
+                    d->d_size, idx);
+          }
+          memcpy(mem->buf + idx, d->d_buf, d->d_size);
+          memset(mem->tags + idx, TAG_ALLOCATED, d->d_size);
+        }
+      }
+    }
+  }
+done:
+  (void)elf_end(e);
+  return rv;
+}
+#endif  /* HAVE_LIBELF */
+
 /*
  * Simple itoa() implementation.  Caller needs to allocate enough
  * space in buf.  Only positive integers are handled.
@@ -700,13 +1119,16 @@ static char *itoa_simple(int n, char *buf, int base)
 
 
 static int fileio_rbin(struct fioparms * fio,
-                  char * filename, FILE * f, unsigned char * buf, int size)
+                  char * filename, FILE * f, AVRMEM * mem, int size)
 {
   int rc;
+  unsigned char *buf = mem->buf;
 
   switch (fio->op) {
     case FIO_READ:
       rc = fread(buf, 1, size, f);
+      if (rc > 0)
+        memset(mem->tags, TAG_ALLOCATED, rc);
       break;
     case FIO_WRITE:
       rc = fwrite(buf, 1, size, f);
@@ -730,7 +1152,7 @@ static int fileio_rbin(struct fioparms * fio,
 
 
 static int fileio_imm(struct fioparms * fio,
-               char * filename, FILE * f, unsigned char * buf, int size)
+               char * filename, FILE * f, AVRMEM * mem, int size)
 {
   int rc = 0;
   char * e, * p;
@@ -753,7 +1175,8 @@ static int fileio_imm(struct fioparms * fio,
                   progname, p);
           return -1;
         }
-        buf[loc++] = b;
+        mem->buf[loc] = b;
+        mem->tags[loc++] = TAG_ALLOCATED;
         p = strtok(NULL, " ,");
         rc = loc;
       }
@@ -777,20 +1200,20 @@ static int fileio_imm(struct fioparms * fio,
 
 
 static int fileio_ihex(struct fioparms * fio, 
-                  char * filename, FILE * f, unsigned char * buf, int size)
+                  char * filename, FILE * f, AVRMEM * mem, int size)
 {
   int rc;
 
   switch (fio->op) {
     case FIO_WRITE:
-      rc = b2ihex(buf, size, 32, 0, filename, f);
+      rc = b2ihex(mem->buf, size, 32, fio->fileoffset, filename, f);
       if (rc < 0) {
         return -1;
       }
       break;
 
     case FIO_READ:
-      rc = ihex2b(filename, f, buf, size);
+      rc = ihex2b(filename, f, mem, size, fio->fileoffset);
       if (rc < 0)
         return -1;
       break;
@@ -807,20 +1230,20 @@ static int fileio_ihex(struct fioparms * fio,
 
 
 static int fileio_srec(struct fioparms * fio,
-                  char * filename, FILE * f, unsigned char * buf, int size)
+                  char * filename, FILE * f, AVRMEM * mem, int size)
 {
   int rc;
 
   switch (fio->op) {
     case FIO_WRITE:
-      rc = b2srec(buf, size, 32, 0, filename, f);
+      rc = b2srec(mem->buf, size, 32, fio->fileoffset, filename, f);
       if (rc < 0) {
         return -1;
       }
       break;
 
     case FIO_READ:
-      rc = srec2b(filename, f, buf, size);
+      rc = srec2b(filename, f, mem, size, fio->fileoffset);
       if (rc < 0)
         return -1;
       break;
@@ -837,8 +1260,38 @@ static int fileio_srec(struct fioparms * fio,
 }
 
 
+#ifdef HAVE_LIBELF
+static int fileio_elf(struct fioparms * fio,
+                      char * filename, FILE * f, AVRMEM * mem,
+                      struct avrpart * p, int size)
+{
+  int rc;
+
+  switch (fio->op) {
+    case FIO_WRITE:
+      fprintf(stderr, "%s: ERROR: write operation not (yet) "
+              "supported for ELF\n",
+              progname);
+      return -1;
+      break;
+
+    case FIO_READ:
+      rc = elf2b(filename, f, mem, p, size, fio->fileoffset);
+      return rc;
+
+    default:
+      fprintf(stderr, "%s: ERROR: invalid ELF file I/O "
+              "operation=%d\n",
+              progname, fio->op);
+      return -1;
+      break;
+  }
+}
+
+#endif
+
 static int fileio_num(struct fioparms * fio,
-	       char * filename, FILE * f, unsigned char * buf, int size,
+	       char * filename, FILE * f, AVRMEM * mem, int size,
 	       FILEFMT fmt)
 {
   const char *prefix;
@@ -883,7 +1336,7 @@ static int fileio_num(struct fioparms * fio,
       if (putc(',', f) == EOF)
 	goto writeerr;
     }
-    num = (unsigned int)buf[i];
+    num = (unsigned int)(mem->buf[i]);
     /*
      * For a base of 8 and a value < 8 to convert, don't write the
      * prefix.  The conversion will be indistinguishable from a
@@ -909,7 +1362,8 @@ static int fileio_num(struct fioparms * fio,
 }
 
 
-int fileio_setparms(int op, struct fioparms * fp)
+int fileio_setparms(int op, struct fioparms * fp,
+                    struct avrpart * p, AVRMEM * m)
 {
   fp->op = op;
 
@@ -935,6 +1389,19 @@ int fileio_setparms(int op, struct fioparms * fp)
       break;
   }
 
+  /*
+   * AVR32 devices maintain their load offset within the file itself,
+   * but AVRDUDE maintains all memory images 0-based.
+   */
+  if ((p->flags & AVRPART_AVR32) != 0)
+  {
+    fp->fileoffset = m->offset;
+  }
+  else
+  {
+    fp->fileoffset = 0;
+  }
+
   return 0;
 }
 
@@ -947,6 +1414,7 @@ static int fmt_autodetect(char * fname)
   int i;
   int len;
   int found;
+  int first = 1;
 
   f = fopen(fname, "r");
   if (f == NULL) {
@@ -956,6 +1424,14 @@ static int fmt_autodetect(char * fname)
   }
 
   while (fgets((char *)buf, MAX_LINE_LEN, f)!=NULL) {
+    /* check for ELF file */
+    if (first &&
+        (buf[0] == 0177 && buf[1] == 'E' &&
+         buf[2] == 'L' && buf[3] == 'F')) {
+      fclose(f);
+      return FMT_ELF;
+    }
+
     buf[MAX_LINE_LEN-1] = 0;
     len = strlen((char *)buf);
     if (buf[len-1] == '\n')
@@ -1003,6 +1479,8 @@ static int fmt_autodetect(char * fname)
         return FMT_SREC;
       }
     }
+
+    first = 0;
   }
 
   fclose(f);
@@ -1017,7 +1495,6 @@ int fileio(int op, char * filename, FILEFMT format,
   int rc;
   FILE * f;
   char * fname;
-  unsigned char * buf;
   struct fioparms fio;
   AVRMEM * mem;
   int using_stdio;
@@ -1030,19 +1507,18 @@ int fileio(int op, char * filename, FILEFMT format,
     return -1;
   }
 
-  rc = fileio_setparms(op, &fio);
+  rc = fileio_setparms(op, &fio, p, mem);
   if (rc < 0)
     return -1;
 
-  /* point at the requested memory buffer */
-  buf = mem->buf;
   if (fio.op == FIO_READ)
     size = mem->size;
 
   if (fio.op == FIO_READ) {
     /* 0xff fill unspecified memory */
-    memset(buf, 0xff, size);
+    memset(mem->buf, 0xff, size);
   }
+  memset(mem->tags, 0, size);
 
   using_stdio = 0;
 
@@ -1066,9 +1542,9 @@ int fileio(int op, char * filename, FILEFMT format,
     if (using_stdio) {
       fprintf(stderr, 
               "%s: can't auto detect file format when using stdin/out.\n"
-              "     Please specify a file format using the -f option and try again.\n", 
-              progname);
-      exit(1);
+              "%s  Please specify a file format and try again.\n", 
+              progname, progbuf);
+      return -1;
     }
 
     format = fmt_autodetect(fname);
@@ -1113,26 +1589,38 @@ int fileio(int op, char * filename, FILEFMT format,
 
   switch (format) {
     case FMT_IHEX:
-      rc = fileio_ihex(&fio, fname, f, buf, size);
+      rc = fileio_ihex(&fio, fname, f, mem, size);
       break;
 
     case FMT_SREC:
-      rc = fileio_srec(&fio, fname, f, buf, size);
+      rc = fileio_srec(&fio, fname, f, mem, size);
       break;
 
     case FMT_RBIN:
-      rc = fileio_rbin(&fio, fname, f, buf, size);
+      rc = fileio_rbin(&fio, fname, f, mem, size);
+      break;
+
+    case FMT_ELF:
+#ifdef HAVE_LIBELF
+      rc = fileio_elf(&fio, fname, f, mem, p, size);
+#else
+      fprintf(stderr,
+              "%s: can't handle ELF file %s, "
+              "ELF file support was not compiled in\n",
+              progname, fname);
+      rc = -1;
+#endif
       break;
 
     case FMT_IMM:
-      rc = fileio_imm(&fio, fname, f, buf, size);
+      rc = fileio_imm(&fio, fname, f, mem, size);
       break;
 
     case FMT_HEX:
     case FMT_DEC:
     case FMT_OCT:
     case FMT_BIN:
-      rc = fileio_num(&fio, fname, f, buf, size, format);
+      rc = fileio_num(&fio, fname, f, mem, size, format);
       break;
 
     default:
@@ -1142,7 +1630,10 @@ int fileio(int op, char * filename, FILEFMT format,
   }
 
   if (rc > 0) {
-    if ((op == FIO_READ) && (strcasecmp(mem->desc, "flash") == 0)) {
+    if ((op == FIO_READ) && (strcasecmp(mem->desc, "flash") == 0 ||
+                             strcasecmp(mem->desc, "application") == 0 ||
+                             strcasecmp(mem->desc, "apptable") == 0 ||
+                             strcasecmp(mem->desc, "boot") == 0)) {
       /*
        * if we are reading flash, just mark the size as being the
        * highest non-0xff byte
