@@ -13,11 +13,10 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: linux_ppdev.h 557 2005-11-29 20:20:22Z joerg_wunsch $ */
+/* $Id: linux_ppdev.h 1198 2013-09-02 20:53:12Z joerg_wunsch $ */
 
 #ifndef linux_ppdev_h
 #define linux_ppdev_h
@@ -25,8 +24,10 @@
 #define OBSOLETE__IOW _IOW
 
 #include <sys/ioctl.h>
+#ifdef HAVE_PARPORT
 #include <linux/parport.h>
 #include <linux/ppdev.h>
+#endif
 
 #include <stdlib.h>
 
