@@ -13,11 +13,10 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
- /* $Id: lists.c 694 2006-12-11 12:47:35Z joerg_wunsch $ */
+ /* $Id: lists.c 1107 2012-11-20 14:03:50Z joerg_wunsch $ */
 
 
 
@@ -478,7 +477,7 @@ lcreat ( void * liststruct, int elements )
 |  at the same time.
  --------------------------------------------------*/
 void 
-ldestroy_cb ( LISTID lid, void (*ucleanup)() )
+ldestroy_cb ( LISTID lid, void (*ucleanup)(void * data_ptr) )
 {
   LIST * l;
   LISTNODE * ln;
@@ -1280,6 +1279,43 @@ lsrch ( LISTID lid, void * p, int (* compare)(void * p1, void * p2) )
   return NULL;
 }
 
+/*----------------------------------------------------------------------
+|  lsort
+|
+|  sort list - sorts list inplace (using bubble sort)
+|
+ ----------------------------------------------------------------------*/
+void
+lsort ( LISTID lid, int (* compare)(void * p1, void * p2) )
+{
+  LIST * l;
+  LISTNODE * lt; /* this */
+  LISTNODE * ln; /* next */
+  int unsorted = 1;
+
+  l = (LIST *)lid;
+
+  CKLMAGIC(l);
+
+  while(unsorted){
+    lt = l->top;
+    unsorted = 0;
+    while (lt!=NULL) {
+      CKMAGIC(lt);
+      ln = lt->next;
+      if (ln!= NULL && compare(lt->data,ln->data) > 0) {
+        void * p = ln->data;
+        ln->data = lt->data;
+        lt->data = p;
+        unsorted = 1;
+      }
+      lt = ln;
+    }
+  }
+
+  CKLMAGIC(l);
+}
+
 
 int lprint ( FILE * f, LISTID lid )
 {
