@@ -14,10 +14,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
-/* $Id: serbb.h 722 2007-01-24 22:43:46Z joerg_wunsch $ */
+/* $Id: serbb.h 1107 2012-11-20 14:03:50Z joerg_wunsch $ */
 
 #ifndef serbb_h
 #define serbb_h
@@ -26,6 +25,7 @@
 extern "C" {
 #endif
 
+extern const char serbb_desc[];
 void serbb_initpgm        (PROGRAMMER * pgm);
 
 #ifdef __cplusplus
