@@ -13,11 +13,10 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: term.c 936 2010-01-22 16:40:17Z joerg_wunsch $ */
+/* $Id: term.c 1211 2013-09-08 19:31:48Z joerg_wunsch $ */
 
 #include "ac_cfg.h"
 
@@ -94,6 +93,9 @@ static int cmd_spi   (PROGRAMMER * pgm, struct avrpart * p,
 static int cmd_pgm   (PROGRAMMER * pgm, struct avrpart * p,
 		      int argc, char *argv[]);
 
+static int cmd_verbose (PROGRAMMER * pgm, struct avrpart * p,
+		      int argc, char *argv[]);
+
 struct command cmd[] = {
   { "dump",  cmd_dump,  "dump memory  : %s <memtype> <addr> <N-Bytes>" },
   { "read",  cmd_dump,  "alias for dump" },
@@ -109,6 +111,7 @@ struct command cmd[] = {
   { "sck",   cmd_sck,   "set <SCK period> (STK500 only)" },
   { "spi",   cmd_spi,   "enter direct SPI mode" },
   { "pgm",   cmd_pgm,   "return to programming mode" },
+  { "verbose", cmd_verbose, "change verbosity" },
   { "help",  cmd_help,  "help" },
   { "?",     cmd_help,  "help" },
   { "quit",  cmd_quit,  "quit" }
@@ -207,13 +210,12 @@ static int chardump_line(char * buffer, unsigned char * p, int n, int pad)
 static int hexdump_buf(FILE * f, int startaddr, unsigned char * buf, int len)
 {
   int addr;
-  int i, n;
+  int n;
   unsigned char * p;
   char dst1[80];
   char dst2[80];
 
   addr = startaddr;
-  i = 0;
   p = (unsigned char *)buf;
   while (len) {
     n = 16;
@@ -761,6 +763,37 @@ static int cmd_pgm(PROGRAMMER * pgm, struct avrpart * p,
   return 0;
 }
 
+static int cmd_verbose(PROGRAMMER * pgm, struct avrpart * p,
+		       int argc, char * argv[])
+{
+  int nverb;
+  char *endp;
+
+  if (argc != 1 && argc != 2) {
+    fprintf(stderr, "Usage: verbose [<value>]\n");
+    return -1;
+  }
+  if (argc == 1) {
+    fprintf(stderr, "Verbosity level: %d\n", verbose);
+    return 0;
+  }
+  nverb = strtol(argv[1], &endp, 0);
+  if (endp == argv[2]) {
+    fprintf(stderr, "%s: can't parse verbosity level \"%s\"\n",
+	    progname, argv[2]);
+    return -1;
+  }
+  if (nverb < 0) {
+    fprintf(stderr, "%s: verbosity level must be positive: %d\n",
+	    progname, nverb);
+    return -1;
+  }
+  verbose = nverb;
+  fprintf(stderr, "New verbosity level: %d\n", verbose);
+
+  return 0;
+}
+
 static int tokenize(char * s, char *** argv)
 {
   int     i, n, l, nargs, offset;
