@@ -14,11 +14,10 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: update.h 722 2007-01-24 22:43:46Z joerg_wunsch $ */
+/* $Id: update.h 1107 2012-11-20 14:03:50Z joerg_wunsch $ */
 
 #ifndef update_h
 #define update_h
@@ -29,6 +28,12 @@ enum {
   DEVICE_VERIFY
 };
 
+enum updateflags {
+  UF_NONE = 0,
+  UF_NOWRITE = 1,
+  UF_AUTO_ERASE = 2,
+};
+
 
 typedef struct update_t {
   char * memtype;
@@ -45,8 +50,9 @@ extern UPDATE * parse_op(char * s);
 extern UPDATE * dup_update(UPDATE * upd);
 extern UPDATE * new_update(int op, char * memtype, int filefmt,
 			   char * filename);
+extern void free_update(UPDATE * upd);
 extern int do_op(PROGRAMMER * pgm, struct avrpart * p, UPDATE * upd,
-		 int nowrite, int verify);
+		 enum updateflags flags);
 
 #ifdef __cplusplus
 }
